function [eigvals, eigvecs] = teig(coefs, n),
  r = length(coefs) - 1;
  # The parameter npoints depends on the desired constant.
  npoints = 1000 + 1;
  xnodes = linspace(0, pi, npoints)';
  lambdas = cosserieval(coefs, xnodes);
  # First part: compute the values of eta at the points xnodes.
  # This part does not depend essentially on n
  # and has complexity C1 * r^3 * npoints, where C1 is a big constant.
  etas = zeros(npoints, 1);
  polcoefs = [flipud(coefs(2 : end)); coefs(1); coefs(2 : end)];
  for j = 1 : npoints,
    polcoefs(r + 1) = coefs(1) - lambdas(j);
    # Todo Optimization 1:
    # the polynomial changes very little from one iteration to another;
    # one could optimize the following computation of the roots:
    z = roots(polcoefs);
    [abszsorted, ind] = sort(abs(z));
    zsorted = z(ind);
    bigroots = zsorted((r + 2) : end);
    es = exp(i * xnodes(j));
    etas(j) = 2 * sum(arg(1 - es / bigroots));
  end
  # Note that eta is defined via the principal value of the argument;
  # therefore it can have jumps (multiples of 2 * pi);
  # we have to remove these jumps in order to make eta continuous
  for j = 2 : npoints,
    jump = round((etas(j - 1) - etas(j)) / (2 * pi));
    etas(j) = etas(j) + 2 * pi * jump;
  end
  # ToDo Optimization 2: it would be better to use splines of higher degree
  etaspline = spline(xnodes, etas);
  # Second part: eigenvalues.
  # This part has complexity C2 n, where C2 depends on the desired precision.
  p = zeros(n, 1);
  nsteps = - log(eps) / log(n + 1) + 1;
  for istep = 1 : nsteps,
    p = (pi * (1 : n)' - ppval(etaspline, p)) / (n + 1);
  end
  eigvals = cosserieval(coefs, p);
  # Third part: eigenvectors.
  # This part has complexity C1 * r^3 * n + C3 * r * n^2.
  if nargout > 1,
    eigvecs = sin(((1 : n)' - (n + 1) / 2) * p' + repmat((1 : n) * pi / 2, n, 1));
    for j = 1 : n,
      polcoefs(r + 1) = coefs(1) - eigvals(j);
      z = roots(polcoefs);
      [abszsorted, ind] = sort(abs(z));
      zsorted = z(ind);
      bigroots = zsorted((r + 2) : end);
      es = exp(i * p(j));
      h = (-1) ^ (r - 1) * poly(bigroots) / prod(bigroots);
      h1 = polyder(h);
      c1 = sin(p(j)) * abs(polyval(h, es));
      bigrootsrep = repmat(transpose(bigroots), n, 1);
      rangenrep = repmat((1 : n)', 1, r - 1);
      coefsofsum = 1 ./ ((bigroots - es) .* (bigroots - 1 / es) .* polyval(h1, bigroots));
      v = ( - (bigrootsrep .^ (- rangenrep)) + (-1) ^ j * (bigrootsrep .^ (rangenrep - n - 1))) * coefsofsum;
      eigvecs(:, j) = eigvecs(:, j) + c1 * v;
      eigvecs(:, j) = eigvecs(:, j) / norm(eigvecs(:, j));
    end  
  end
end

function v = cosserieval(coefs, x),
  r = length(coefs) - 1;
  xmultiplies = x * (1 : r);
  v = coefs(1) * ones(size(x)) + 2 * cos(xmultiplies) * coefs(2 : end);
end

# coefs and x must be column-vectors of real numbers
# Example: if coefs = [a b c d], then it returns
# a + 2 * b * cos(x) + 2 * c * cos(2 * x) + 2 * d * cos(3 * x)


# These programas are based on the articles by A. B\"{o}ttcher, S. Grudsky and E. Maksimenko
# http://dx.doi.org/10.1016/j.cam.2009.10.010
# http://dx.doi.org/10.1007/978-3-0346-0548-9_2

